"""
Created on Feb 16 2018

@author: MCC
"""
from ctypes import c_uint, c_double, c_longlong, byref
from .ul_enums import AiInputMode, AInFlag, AInScanFlag, Range, ScanOption, ScanStatus, TriggerType, WaitType
from .ul_structs import AiQueueElement, TransferStatus
from .ul_exception import ULException
from .ul_c_interface import lib
from .ai_info import AiInfo
from .ai_config import AiConfig


class AiDevice:
    """
    Analog input subsystem of the UL DAQ Device.

    Args:
        handle: UL DAQ Device handle.
    """

    def __init__(self, handle):
        self.__handle = handle
        self.__ai_info = AiInfo(handle)
        self.__ai_config = AiConfig(handle)

    def get_info(self):
        # type: () -> AiInfo
        """
        Gets the analog input information object for the UL DAQ Device.

        Returns:
            :class:`AiInfo` object used for retrieving configuration
            information about the analog input subsystem of the UL DAQ Device.
        """
        return self.__ai_info
    
    def get_config(self):
        # type: () -> AiConfig
        """
        Gets analog input configuration object for the UL DAQ Device.

        Returns:
            :class:`AiConfig` object used for retrieving configuration
            information about the analog input subsystem of the UL DAQ Device.
        """
        return self.__ai_config
    
    def a_in(self, channel, input_mode, voltage_range, flags):
        # type: (int, AiInputMode, Range, AInFlag) -> float
        """
        Returns the value read from an A/D channel.

        Args:
            channel: A/D channel number.
            input_mode: :class:`AiInputMode`
            voltage_range: :class:`Range`
            flags: A bitmask specifying whether to scale and/or calibrate
                data. Possible options are enumerated by :class:`AInFlag`.


        Returns:
            The value of the A/D channel.

        Raises:
            :class:`ULException`
        """
        data = c_double()
        err = lib.ulAIn(self.__handle, channel, input_mode, voltage_range, flags, byref(data))
        if err != 0:
            raise ULException(err)
        return data.value
    
    def a_in_scan(self, low_channel, high_channel, input_mode, voltage_range, samples_per_channel, rate, options, flags,
                  data):
        # type: (int, int, AiInputMode, Range, int, float, ScanOption, AInScanFlag, list[float]) -> float
        """
        Scans a range of A/D channels, and stores the samples.

        Args:
            low_channel: First A/D channel in the scan.
            high_channel: Last A/D channel in the scan.
            input_mode: :class:`AiInputMode`
            voltage_range: :class:`Range`.
            samples_per_channel: the number of A/D samples to collect from
                each channel in the scan.
            rate: A/D sample rate in samples per channel per second.
            options: A bitmask that specifies A/D scan options. Possible
                options are enumerated by :class:`ScanOption`.
            flags: A bitmask specifying whether to scale and/or calibrate
                data. Possible options are enumerated by
                :class:`AInScanFlag`.
            data: Pointer to the buffer to receive the data.

        Returns:
            The actual input scan rate of the scan.

        Raises:
            :class:`ULException`
        """
        rate = c_double(rate)
        err = lib.ulAInScan(self.__handle, low_channel, high_channel, input_mode, voltage_range,
                            samples_per_channel, byref(rate), options, flags, data)
        if err != 0:
            raise ULException(err)
        return rate.value

    def a_in_load_queue(self, queue):
        # type: (AiQueueElement) -> None
        """
        Loads the A/D queue of a specified device.

        Args:
            queue: An array of :class:`AiQueueElement` structs, each of which
                contains fields specifying the channel, range, and mode.

        Raises:
            :class:`ULException`
        """
        num_elements = len(queue)
        err = lib.ulAInLoadQueue(self.__handle, queue, num_elements)
        if err != 0:
            raise ULException(err)
   
    def set_trigger(self, trig_type, trig_chan, level, variance, retrigger_sample_count):
        # type: (TriggerType, int, float, float, int) -> None
        """
        Configures the trigger parameters that will be used when
        a_in_scan() is called with the RETRIGGER or EXTTRIGGER
        :class:`ScanOption`

        Args:
            trig_type: :class:`TriggerType` based on the external
                trigger source.
            trig_chan: The trigger channel; ignored if trig_type is set to
                POS_EDGE, NEG_EDGE, HIGH, LOW, GATE_HIGH, GATE_LOW, RISING,
                or FALLING.
            level: The level at or around which the trigger event should be
                detected; ignored if trig_type is set to POS_EDGE, NEG_EDGE,
                HIGH, LOW, GATE_HIGH, GATE_LOW, RISING, or FALLING.
            variance: The degree to which the input signal can vary
                relative to the level parameter; ignored for all
                types where level is ignored. For pattern triggering,
                this argument serves as the mask value.
            retrigger_sample_count: The number of samples per trigger
                to acquire with each trigger event; ignored unless the
                RETRIGGER ScanOption is set for the scan.

        Raises:
            :class:`ULException`
        """
        err = lib.ulAInSetTrigger(self.__handle, trig_type, trig_chan, level, variance,
                                  retrigger_sample_count)
        if err != 0:
            raise ULException(err)
    
    def get_scan_status(self):
        # type: () -> tuple[ScanStatus, TransferStatus]
        """
        Gets the status, count, and index of an A/D scan operation.

        Returns:
            A tuple containing the scan status :class:`ScanStatus`
            and transfer status :class:`TransferStatus` for the
            analog input background operation.

        Raises:
            :class:`ULException`
        """
        status = c_uint()
        transfer_status = TransferStatus()
        err = lib.ulAInScanStatus(self.__handle, byref(status), transfer_status)
        if err != 0:
            raise ULException(err)
        return ScanStatus(status.value), transfer_status
    
    def scan_stop(self):
        # type: () -> None
        """
        Stops the analog input operation currently running.

        Raises:
            :class:`ULException`
        """
        err = lib.ulAInScanStop(self.__handle)
        if err != 0:
            raise ULException(err)

    def scan_wait(self, wait_type, timeout):
        # type: (WaitType, float) -> None
        """
        Waits until the scan operation completes or the specified timeout elapses.

        Args:
            wait_type: The :class:`WaitType`.
            timeout: The timeout value in seconds (s); set to -1 to
                wait indefinitely for the scan function to end.

        Raises:
            :class:`ULException`
        """
        wait_param = c_longlong(0)
        err = lib.ulAInScanWait(self.__handle, wait_type, wait_param, timeout)
        if err != 0:
            raise ULException(err)
