"""
Created on Feb 16 2018

@author: MCC
"""


from ctypes import c_uint, c_double, byref
from .ul_enums import TriggerType, ScanStatus, WaitType
from .ul_structs import DaqInChanDescriptor, TransferStatus
from .ul_exception import ULException
from .ul_c_interface import lib
from .daqi_info import DaqiInfo


def daqi_chan_descriptor_array(size, descriptor_list):
    chan_descriptor_array = DaqInChanDescriptor * size  # type: type
    return chan_descriptor_array(*descriptor_list)


class DaqiDevice:
    """
    Daq Input subsystem of the UL DAQ Device.

    Args:
        handle: UL DAQ Device handle.
    """

    def __init__(self, handle):
        self.__handle = handle
        self.__daqi_info = DaqiInfo(handle)

    def get_info(self):
        # type: () -> DaqiInfo
        """
        Gets the daq input information object for the UL DAQ Device.

        Returns:
            A :class:`DaqiInfo` object used for retrieving
            information about the DAQ input subsystem of the UL DAQ Device.
        """
        return self.__daqi_info
    
    def set_trigger(self, trig_type, trig_chan, level, variance, retrigger_sample_count):
        # type: (TriggerType, int, float, float, int) -> None
        """
        Configures the trigger parameters that will be used when
        daq_in_scan() is called with the RETRIGGER or EXTTRIGGER
        ScanOption.

        Args:
            trig_type: The type of triggering :class:`TriggerType`
                based on the external trigger source.
            trig_chan: A :class:`DaqInChanDescriptor` object.
            level: The level at or around which the trigger event should be detected;
                ignored if trig_type is set to POS_EDGE, NEG_EDGE, HIGH, LOW, GATE_HIGH,
                GATE_LOW, RISING, or FALLING.
            variance: The degree to which the input signal can vary relative to the
                level parameter; ignored for all types where level is ignored. For
                pattern triggering, this argument serves as the mask value.
            retrigger_sample_count: The number of samples per trigger to acquire
                with each trigger event; ignored unless the RETRIGGER ScanOption
                is set for the scan.

        Raises:
            :class:`ULException`
        """
        err = lib.ulDaqInSetTrigger(self.__handle, trig_type, trig_chan, level, variance, retrigger_sample_count)
        if err != 0:
            raise ULException(err)
    
    def daq_in_scan(self, channel_descriptors, samples_per_channel, rate, options, flags, data):
        # type: (list[DaqInChanDescriptor], int, float, int, int, list[float]) -> float
        """
        Allows scanning of multiple input subsystems, such as analog, digital,
        counter, and stores the samples in an array. This method only works
        with devices that support synchronous input.

        Args:
            channel_descriptors: An array of :class:`DaqInChanDescriptor` structs.
            samples_per_channel: the number of samples to collect from each
                channel in the scan.
            rate: Sample input rate in samples per second.
            options: A bitmask that specifies scan options. Possible options
                are enumerated by :class:`ScanOption`.
            flags: A bitmask that specifies whether to scale and/or calibrate
                data. Possible options are enumerated by
                :class:`DaqInScanFlag`.
            data: The data buffer to receive the data being read.

        Returns:
            The actual input scan rate of the scan.

        Raises:
            :class:`ULException`
        """
        num_channels = len(channel_descriptors)
        rate = c_double(rate)
        chan_descriptor_array = daqi_chan_descriptor_array(num_channels, channel_descriptors)
        err = lib.ulDaqInScan(self.__handle, chan_descriptor_array, num_channels, samples_per_channel, byref(rate),
                              options, flags, data)
        if err != 0:
            raise ULException(err)
        return rate.value
        
    def get_scan_status(self):
        # type: () -> tuple[ScanStatus, TransferStatus]
        """
        Gets the current status of the synchronous input operation.

        Returns:
            A tuple containing the scan status :class:`ScanStatus`
            and transfer status :class:`TransferStatus` for the
            daq input background operation.

        Raises:
            :class:`ULException`
        """
        status = c_uint()
        transfer_status = TransferStatus()
        err = lib.ulDaqInScanStatus(self.__handle, byref(status), transfer_status)
        if err != 0:
            raise ULException(err)
        return ScanStatus(status.value), transfer_status
    
    def scan_stop(self):
        # type: () -> None
        """
        Stops the synchronous input operation currently running.

        Raises:
            :class:`ULException`
        """
        err = lib.ulDaqInScanStop(self.__handle)
        if err != 0:
            raise ULException(err)

    def scan_wait(self, wait_type, timeout):
        # type: (WaitType, float) -> None
        """
        Waits until the scan operation completes or the specified timeout elapses.

        Args:
            wait_type: The :class:`WaitType`.
            timeout: The timeout value in seconds (s); set to -1 to
                wait indefinitely for the scan function to end.

        Raises:
            :class:`ULException`
        """
        err = lib.ulDaqInScanWait(self.__handle, wait_type, None, timeout)
        if err != 0:
            raise ULException(err)
