"""
Created on Feb 16 2018

@author: MCC
"""
from ctypes import c_double, byref, c_bool, c_uint, c_ulonglong
from .ul_exception import ULException
from .ul_c_interface import lib
from .ul_enums import DigitalPortType, DigitalDirection, DInScanFlag, DOutScanFlag, ScanOption, ScanStatus, TriggerType
from .ul_enums import WaitType
from .ul_structs import TransferStatus

from .dio_info import DioInfo
from .dio_config import DioConfig


class DioDevice:
    """
    Digital I/O subsystem of the UL DAQ Device.

    Args:
        handle: UL DAQ Device handle.
    """

    def __init__(self, handle):
        self.__handle = handle
        self.__dio_info = DioInfo(handle)
        self.__dio_config = DioConfig(handle)

    def get_info(self):
        # type: () -> DioInfo
        """
        Get Digital I/O Information object for the UL DAQ Device.

        Returns:
            A :class:`DioInfo` object used for retrieving information about the
            digital I/O subsystem of the UL DAQ Device.
        """
        return self.__dio_info

    def get_config(self):
        # type: () -> DioConfig
        """
        Get Digital I/O Configuration object for the UL DAQ Device.

        Returns:
            A :class:`DioConfig` object used for retrieving configuration
            information about the digital I/O subsystem of the UL DAQ Device.
        """
        return self.__dio_config

    def d_config_port(self, port_type, direction):
        # type: (DigitalPortType, DigitalDirection) -> None
        """
        Configures a digital port as input or output.

        Args:
            port_type: The digital port :class:`DigitalPortType`;
                the port must be configurable.
            direction: The port direction :class:`DigitalDirection`
                (input or output).

        Raises:
            :class:`ULException`
        """

        err = lib.ulDConfigPort(self.__handle, port_type, direction)
        if err != 0:
            raise ULException(err)

    def d_config_bit(self, port_type, bit_number, direction):
        # type: (DigitalPortType, int, DigitalDirection) -> None
        """
        Configures a digital bit as input or output.

        Args:
            port_type: The digital port containing the bit to configure
                :class:`DigitalPortType`;the bit must be configurable.
            bit_number: The bit number within the specified port.
            direction: The bit direction :class:`DigitalDirection`
                (input or output).

        Raises:
            :class:`ULException`
        """
        err = lib.ulDConfigBit(self.__handle, port_type, bit_number, direction)
        if err != 0:
            raise ULException(err)

    def d_in(self, port_type):
        # type: (DigitalPortType) -> int
        """
        Returns the value read from a digital port.

        Args:
            port_type: Digital port to read :class:`DigitalPortType`.

        Returns:
            The port value.

        Raises:
            :class:`ULException`
        """
        data = c_ulonglong()

        err = lib.ulDIn(self.__handle, port_type, byref(data))
        if err != 0:
            raise ULException(err)
        return data.value

    def d_out(self, port_type, data):
        # type: (DigitalPortType, int) -> None
        """
        Writes the specified value to a digital output port.

        Args:
            port_type: Digital port type :class:`DigitalPortType`.
            data: The port value.

        Raises:
            :class:`ULException`
        """
        err = lib.ulDOut(self.__handle, port_type, data)
        if err != 0:
            raise ULException(err)

    def d_bit_in(self, port_type, bit_number):
        # type: (DigitalPortType, int) -> int
        """
        Returns the value of a digital bit.

        Args:
            port_type: Digital port :class:`DigitalPortType`
                containing the bit to be read.
            bit_number: The bit position within the port to read.

        Returns:
            The bit value.

        Raises:
            :class:`ULException`
        """
        data = c_uint()

        err = lib.ulDBitIn(self.__handle, port_type, bit_number, byref(data))
        if err != 0:
            raise ULException(err)
        return c_bool(data).value

    def d_bit_out(self, port_type, bit_number, data):
        # type: (DigitalPortType, int, int) -> None
        """
        Writes a value to a digital bit.

        Args:
            port_type: Digital port :class:`DigitalPortType`
                containing the bit to be written.
            bit_number: The number of the bit to be written.
            data: The bit value to write.

        Raises:
            :class:`ULException`
        """
        err = lib.ulDBitOut(self.__handle, port_type, bit_number, data)
        if err != 0:
            raise ULException(err)

    def d_in_scan(self, low_port_type, high_port_type, samples_per_port, rate, options, flags, data):
        # type: (DigitalPortType, DigitalPortType, int, float, ScanOption, DInScanFlag, list[int]) -> float
        """
        Reads a range of digital ports.

        Args:
            low_port_type: First port :class:`DigitalPortType` in the
                scan.
            high_port_type: Last port :class:`DigitalPortType` in the
                scan.
            samples_per_port: The number of samples to read.
            rate: Sample input rate in samples per second.
            options: A bitmask specifying scan options. Possible options are
                enumerated by :class:`ScanOption`.
            flags: A bitmask specifying data options for the scan.  Possible
                options are enumerated by :class:`DInScanFlag`.
            data: The buffer to receive the digital data.

        Returns:
            The actual sample input rate of the scan.

        Raises:
            :class:`ULException`
        """
        rate = c_double(rate)
        err = lib.ulDInScan(self.__handle, low_port_type, high_port_type,  samples_per_port, byref(rate),
                            options, flags, data)
        if err != 0:
            raise ULException(err)
        return rate.value

    def d_out_scan(self, low_port_type, high_port_type, samples_per_port, rate, options, flags, data):
        # type: (DigitalPortType, DigitalPortType, int, float, ScanOption, DOutScanFlag, list[int]) -> float
        """
        Writes data to a digital port.

        Args:
            low_port_type: First port :class:`DigitalPortType` in the
                scan.
            high_port_type: Last port :class:`DigitalPortType` in the
                scan.
            samples_per_port: The number of samples per port to write.
            rate: Sample output rate in samples per second.
            options: A bitmask specifying scan options.  Possible options are
                enumerated by :class:`ScanOption`.
            flags: A bitmask specifying data options for the scan. Possible
                options are enumerated by :class:`DInScanFlag`.
            data: The data buffer to be written.

        Returns:
            The actual sample output rate of the scan.

        Raises:
            :class:`ULException`
        """
        rate = c_double(rate)
        err = lib.ulDOutScan(self.__handle, low_port_type, high_port_type, samples_per_port, byref(rate),
                             options, flags, data)
        if err != 0:
            raise ULException(err)
        return rate.value

    def d_in_set_trigger(self, trig_type, trig_chan, level, variance, retrigger_sample_count):
        # type: (TriggerType, int, float, float, int) -> None
        """
        Configures the trigger parameters that will be used when d_in_scan()
        is called with the RETRIGGER or EXTTRIGGER ScanOption.

        Args:
            trig_type: :class:`TriggerType` based on the external
                trigger source.
            trig_chan: The trigger channel; ignored if trig_type is set
                to POS_EDGE, NEG_EDGE, HIGH, LOW, GATE_HIGH, GATE_LOW,
                RISING, or FALLING.
            level: The level at or around which the trigger event should be
                detected; ignored if trig_type is set to POS_EDGE, NEG_EDGE,
                HIGH, LOW, GATE_HIGH, GATE_LOW, RISING, or FALLING.
            variance: The degree to which the input signal can vary relative
                to the level parameter; ignored for all types where level is
                ignored. For pattern triggering, this argument serves as the
                mask value.
            retrigger_sample_count: The number of samples per trigger to
                acquire with each trigger event; ignored unless the RETRIGGER
                ScanOption is set for the scan.

        Raises:
            :class:`ULException`
        """
        err = lib.ulDInSetTrigger(self.__handle, trig_type, trig_chan, level, variance, retrigger_sample_count)
        if err != 0:
            raise ULException(err)

    def d_out_set_trigger(self, trig_type, trig_chan, level, variance, retrigger_sample_count):
        # type: (TriggerType, int, float, float, int) -> None
        """
        Configures the trigger parameters that will be used when d_out_scan()
        is called with the RETRIGGER or EXTTRIGGER ScanOption.

        Args:
            trig_type: :class:`TriggerType` based on the external
                trigger source.
            trig_chan: The trigger channel; ignored if trig_type is set
                to POS_EDGE, NEG_EDGE, HIGH, LOW, GATE_HIGH, GATE_LOW,
                RISING, or FALLING.
            level: The level at or around which the trigger event should be
                detected; ignored if trig_type is set to POS_EDGE, NEG_EDGE,
                HIGH, LOW, GATE_HIGH, GATE_LOW, RISING, or FALLING.
            variance: The degree to which the input signal can vary relative
                to the level parameter; ignored for all types where level is
                ignored. For pattern triggering, this argument serves as the
                mask value.
            retrigger_sample_count: The number of samples per trigger to
                acquire with each trigger event; ignored unless the RETRIGGER
                ScanOption is set for the scan.

        Raises:
            :class:`ULException`
        """
        err = lib.ulDOutSetTrigger(self.__handle, trig_type, trig_chan, level, variance, retrigger_sample_count)
        if err != 0:
            raise ULException(err)

    def d_in_get_scan_status(self):
        # type: () -> tuple[ScanStatus, TransferStatus]
        """
        Gets the status of the digital input scan operation.

        Returns:
            A tuple containing the scan status :class:`ScanStatus`
            and transfer status :class:`TransferStatus` of the
            digital input background operation.

        Raises:
            :class:`ULException`
        """
        status = c_uint()
        transfer_status = TransferStatus()

        err = lib.ulDInScanStatus(self.__handle, byref(status), transfer_status)
        if err != 0:
            raise ULException(err)
        return ScanStatus(status.value), transfer_status

    def d_out_get_scan_status(self):
        # type: () -> tuple[ScanStatus, TransferStatus]
        """
        Gets the status of the digital output scan operation.

        Returns:
            A tuple containing the scan status :class:`ScanStatus`
            and transfer status :class:`TransferStatus` of the
            digital output background operation.

        Raises:
            :class:`ULException`
        """
        status = c_uint()
        transfer_status = TransferStatus()

        err = lib.ulDOutScanStatus(self.__handle, byref(status), transfer_status)
        if err != 0:
            raise ULException(err)
        return ScanStatus(status.value), transfer_status

    def d_in_scan_stop(self):
        # type: () -> None
        """
        Stops the digital input operation currently running.

        Raises:
            :class:`ULException`
        """
        err = lib.ulDInScanStop(self.__handle)
        if err != 0:
            raise ULException(err)

    def d_out_scan_stop(self):
        # type: () -> None
        """
        Stops the digital output operation currently running.

        Raises:
            :class:`ULException`
        """
        err = lib.ulDOutScanStop(self.__handle)
        if err != 0:
            raise ULException(err)

    def d_in_scan_wait(self, wait_type, timeout):
        # type: (WaitType, float) -> None
        """
        Waits until the scan operation completes or the specified timeout elapses.

        Args:
            wait_type: The :class:`WaitType`.
            timeout: The timeout value in seconds (s); set to -1 to
                wait indefinitely for the scan function to end.

        Raises:
            :class:`ULException`
        """
        err = lib.ulDInScanWait(self.__handle, wait_type, None, timeout)
        if err != 0:
            raise ULException(err)

    def d_out_scan_wait(self, wait_type, timeout):
        # type: (WaitType, float) -> None
        """
        Stops a background output operation for a specified time.

        Args:
            wait_type: The :class:`WaitType`.
            timeout: The timeout value in milliseconds (ms).

        Raises:
            :class:`ULException`
        """
        err = lib.ulDOutScanWait(self.__handle, wait_type, None, timeout)
        if err != 0:
            raise ULException(err)
